﻿using Agility.Routines;
using Agility.Settings;
using Buddy.Wildstar.BotCommon;
using Buddy.Wildstar.Engine;
using Buddy.Wildstar.Game;
using log4net;
using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Agility
{
    class Agility : IBot, IUIButtonProvider
    {
        public string Author { get { return "Deathdisguise"; } }
        public string Name { get { return "Agility"; } }
        public string Version { get { return "0.$LastChangedRevision: $"; } }

        #region IBot Implementation

        private bool _CanBePulsed = false;

        public void OnInitialize()
        {
        }
        public void OnUninitialize()
        {
        }
        public void OnRegistered()
        {
        }
        public void OnUnregistered()
        {
        }
        public void OnStart()
        {
            _CanBePulsed = true;

            VariableController.SaveSettings();
            VariableController.LoadPreferredSettings();

            GameEngine.BotPulsator.TicksPerSecond = AgilitySettings.TPS;

            // Register Event Handlers.
            GameManager.Lua.Events.RegisterEvent(LuaEvent.CanVacuumChange, HandleAutoLoot);
            GameManager.Lua.Events.RegisterEvent(LuaEvent.ActivateCCStateStun, HandleAutoBreak);
            GameManager.Lua.Events.RegisterEvent(LuaEvent.StunVGPressed, HandleAutoBreak);
            GameManager.Lua.Events.RegisterEvent(LuaEvent.AbilityBookChange, HandleRefresh);
            GameManager.Lua.Events.RegisterEvent(LuaEvent.LogOut, HandleRefresh);
            // Cast Monitoring Events
            // Threshold for Charged Shot
            GameManager.Lua.Events.RegisterEvent(LuaEvent.StartSpellThreshold, HandleThresholdStart);
            GameManager.Lua.Events.RegisterEvent(LuaEvent.UpdateSpellThreshold, HandleThresholdUpdate);
            GameManager.Lua.Events.RegisterEvent(LuaEvent.ClearSpellThreshold, HandleThresholdClear);

        }
        public void OnStop()
        {
            _CanBePulsed = false;

            VariableController.ResetSettings();

            // Unregister Event Handlers.
            GameManager.Lua.Events.UnregisterEvent(LuaEvent.CanVacuumChange, HandleAutoLoot);
            GameManager.Lua.Events.UnregisterEvent(LuaEvent.ActivateCCStateStun, HandleAutoBreak);
            GameManager.Lua.Events.UnregisterEvent(LuaEvent.StunVGPressed, HandleAutoBreak);
            GameManager.Lua.Events.UnregisterEvent(LuaEvent.AbilityBookChange, HandleRefresh);
            GameManager.Lua.Events.UnregisterEvent(LuaEvent.LogOut, HandleRefresh);
            // Cast Monitoring Events
            // Threshold for Charged Shot
            GameManager.Lua.Events.UnregisterEvent(LuaEvent.StartSpellThreshold, HandleThresholdStart);
            GameManager.Lua.Events.UnregisterEvent(LuaEvent.UpdateSpellThreshold, HandleThresholdUpdate);
            GameManager.Lua.Events.UnregisterEvent(LuaEvent.ClearSpellThreshold, HandleThresholdClear);
        }
        public bool CanBePulsed
        {
            get { return _CanBePulsed; }
        }
        public void Pulse()
        {
            using (GameManager.Memory.AcquireFrame(true))
            {
                DoWork();
                GameManager.Lua.Events.Pulse();
            }
        }
        public bool Equals(IBot other)
        {
            if (this.Name != other.Name)
                return false;

            if (this.Version != other.Version)
                return false;

            if (this.Author != other.Author)
                return false;

            return true;
        }

        #endregion

        #region IUIButtonProvider

        public string ButtonText
        {
            get { return "AGILITY"; }
        }
        public void OnButtonClicked(object sender)
        {
            AgilitySettings.Toggle();
        }

        #endregion

        private static ILog Log = LogManager.GetLogger("Agility");
        private AgilitySettings AgilitySettings = AgilitySettings.Instance;

        [DllImport("user32.dll")]
        private static extern short GetAsyncKeyState(System.Windows.Forms.Keys vKey);
        [DllImport("user32.dll")]
        private static extern IntPtr GetForegroundWindow();

        private void DoWork()
        {
            if (CanFire(Keys.D1))
                SpellController.Cast((Activator.CreateInstance(Type.GetType(AgilitySettings.Routine1)) as IAgilityRoutine).Routine());

            if (CanFire(Keys.D2))
                SpellController.Cast((Activator.CreateInstance(Type.GetType(AgilitySettings.Routine2)) as IAgilityRoutine).Routine());

            if (CanFire(Keys.D3))
                SpellController.Cast((Activator.CreateInstance(Type.GetType(AgilitySettings.Routine3)) as IAgilityRoutine).Routine());

            if (CanFire(Keys.D4))
                SpellController.Cast((Activator.CreateInstance(Type.GetType(AgilitySettings.Routine4)) as IAgilityRoutine).Routine());

            if (CanFire(Keys.D5))
                SpellController.Cast((Activator.CreateInstance(Type.GetType(AgilitySettings.Routine5)) as IAgilityRoutine).Routine());

            if (CanFire(Keys.D6))
                SpellController.Cast((Activator.CreateInstance(Type.GetType(AgilitySettings.Routine6)) as IAgilityRoutine).Routine());

            if (CanFire(Keys.D7))
                SpellController.Cast((Activator.CreateInstance(Type.GetType(AgilitySettings.Routine7)) as IAgilityRoutine).Routine());

            if (CanFire(Keys.D8))
                SpellController.Cast((Activator.CreateInstance(Type.GetType(AgilitySettings.Routine8)) as IAgilityRoutine).Routine());
        }

        private bool CanFire(Keys key)
        {
            // Key hasn't been pressed.
            if (GetAsyncKeyState(key) == 0)
                return false;

            // Not focused? Do nothing.
            if (AgilitySettings.IsForegroundRequired && GameManager.Memory.Process.MainWindowHandle != GetForegroundWindow())
                return false;

            // Don't do anything unless you're loaded into game.
            if (GameManager.WorldLoadState != WorldLoadState.InWorld)
                return false;

            // Don't do anything if a window is open.
            if ((int)GameManager.CurrentOpenWindowType != 99)
                return false;

            return true;
        }

        private void HandleAutoLoot(string eventname, LuaEventArgument[] args)
        {
            bool on = args[0];

            if (!on)
                return;

            if (AgilitySettings.AutoLoot && (AgilitySettings.GroupAutoLoot || (!AgilitySettings.GroupAutoLoot && !GameManager.LocalPlayer.IsInGroup)))
            {
                if (GameManager.Inventory.Bags.FreeSlots + 1 > 0)
                {
                    Log.Info("Looting...");
                    CommonBehaviors.VacuumLoot();
                }
            }
        }

        private void HandleRefresh(string eventname, LuaEventArgument[] args)
        {
            Log.Info("Refreshing abilities...");
            GameManager.ForceUpdate();
            SpellCache.Refresh();
        }

        private void AutoCSI()
        {
            // Don't do anything unless you're loaded into game.
            if (GameManager.WorldLoadState != WorldLoadState.InWorld)
                return;
        }

        private void HandleAutoBreak(string eventname, LuaEventArgument[] args)
        {
            if (!AgilitySettings.AutoBreak)
                return;

            // Credit: Apoc.
            switch (GameManager.CCStunDirection)
            {
                case CCStunDirection.Backward:
                    GameManager.InputManager.KeyPress(InputAction.StunBreakoutDown);
                    break;
                case CCStunDirection.Forward:
                    GameManager.InputManager.KeyPress(InputAction.StunBreakoutUp);
                    break;
                case CCStunDirection.Left:
                    GameManager.InputManager.KeyPress(InputAction.StunBreakoutLeft);
                    break;
                case CCStunDirection.Right:
                    GameManager.InputManager.KeyPress(InputAction.StunBreakoutRight);
                    break;
            }
        }
        // Threshold Handlers used with charged shot
        private void HandleThresholdStart(string eventname, LuaEventArgument[] args)
        {
            Log.Info("Starting Threshold...");
            SpellController.CastBar.ThresholdStart(args);
        }
        private void HandleThresholdUpdate(string eventname, LuaEventArgument[] args)
        {
            Log.Info("Updating Threshold...");
            SpellController.CastBar.ThresholdUpdate(args);
        }
        private void HandleThresholdClear(string eventname, LuaEventArgument[] args)
        {
            Log.Info("Clearing Threshold...");
            SpellController.CastBar.ThresholdClear(args);
        }
    }
}
