﻿using Buddy.Common;
using Buddy.Wildstar.BotCommon;
using Buddy.Wildstar.Game;
using log4net;
using System;
using System.Linq;
using Spell = Buddy.Wildstar.Game.SpellManager.SpellInformation;

namespace Agility
{
    static class SpellController
    {
        private static ILog Log = LogManager.GetLogger("Agility [SpellController]");

        public static int BuffCount(string buffName)
        {
            return GameManager.Lua.GetReturnValue<int>(
              @"for _,buff in pairs(GameLib.GetPlayerUnit():GetBuffs().arBeneficial) do
                    if ( buff.splEffect:GetName() == '" + buffName + @"') then
                        return buff.nCount
                    end
                end
                return 0
            ");
        }

        public static bool CanCast(string spellName, bool instant = false)
        {
            AgilitySpell spell = SpellCache.GetSpell(spellName);
            if (!spell.IsValid)
                return false;

            if (spell.CooldownFlags == CastResult.Ok) // OK
                return true;

            return false;

            /*switch ((CastResult)ability.SpellInfo.CooldownFlags)
            {
                case CastResult.CasterVitalCostResource0: // ???.
                case CastResult.CasterVitalCostResource1: // Kinetic Energy.
                case CastResult.CasterVitalCostResource3: // Suit Power.
                case CastResult.CasterVitalCostResource5: // Actuators.
                case CastResult.SpellNoCharges: // No charges.
                case CastResult.Prereq_CasterCast: // Already casting.
                    return false;
            
                case CastResult.SpellCooldown:
                case CastResult.SpellGroupCooldown:
                    return false;
            
                case CastResult.SpellGlobalCooldown:
                    return false;
            
                default:
                    return true;
            }*/
        }

        public static void Cast(string spellName)
        {
            AgilitySpell spell = SpellCache.GetSpell(spellName);
            if (!spell.IsValid)
                return;

            Log.Info(String.Format("[{0}] [{1}] [{2}] \t{3}",
                spell.TierId,
                spell.Tier,
                spell.CooldownFlags,
                spellName
            ));

            if (spell.Name == "Spell Surge")
                CastTracker.Surge = DateTime.Now;

            if (spell.Name == "Ignite")
                CastTracker.Ignite = DateTime.Now;

            spell.Cast();
        }

        public static int Cooldown(string spellName)
        {
            AgilitySpell spell = SpellCache.GetSpell(spellName);
            if (!spell.IsValid)
                return 10000;

            return spell.CooldownRemaining;
        }

        public static bool HasAMP(string AugName)
        {
            // Credit : https://www.thebuddyforum.com/members/337126-lukov.html
            return GameManager.Lua.GetReturnValue<bool>(
              @"for _,AMP in pairs(AbilityBook.GetEldanAugmentationData(AbilityBook.GetCurrentSpec()).tAugments) do
                    if (AMP.strTitle == '" + AugName + @"') then
                        return (AMP.eEldanAvailability == AbilityBook.CodeEnumEldanAvailability.Activated)
                    end
                end
                return false
            ");
        }

        public static bool HasSpell(string spellName)
        {
            AgilitySpell spell = SpellCache.GetSpell(spellName);
            if (!spell.IsValid)
                return false;

            return spell.Tier > 0;
        }

        public static bool IsTicking(string effectName)
        {
            return GameManager.Actors.Values.Any(a => a.Buffs.Any(b => b.Name == effectName));
        }

        public static bool IsTickingID(int effectID)
        {
            return GameManager.Actors.Values.Any(a => a.Buffs.Any(b => b.SpellId == effectID));
        }

        public static void SetStance(string spell)
        {
            GameManager.Lua.DoString(
              @"for idx, spellObject in pairs(GameLib.GetClassInnateAbilitySpells().tSpells) do
                    if idx % 2 == 0 and spellObject:GetName() == '" + spell + @"' then
                        GameLib.SetCurrentClassInnateAbilityIndex(idx/2)
                    end
                end
            ");
        }

        public static void Release(string spellName, int chargeCount)
        {
            if (!GameManager.LocalPlayer.IsCasting)
                return;

            int threshold = (int)GameManager.Lua.GetReturnValue<float>("return GameLib.GetPlayerUnit():GetCastElapsed()");
            if (threshold >= 800 * chargeCount)
                SpellManager.GetSpell(spellName).Cast(false);

        }

        public static int Tier(string spellName)
        {
            AgilitySpell spell = SpellCache.GetSpell(spellName);
            if (!spell.IsValid)
                return 0;

            return spell.Tier;
        }

        public static class CastTracker
        {
            public static DateTime Ignite = DateTime.Now;
            public static DateTime Surge = DateTime.Now;
        }


            public static class CastBar
        {
            public static DateTime CastStartTime;
            public static int SpellID;
            public static string SpellName;
            public static int CurrentThresshold;
            public static int MaxThresshold;

            public static bool ThresholdStart(LuaEventArgument[] args)
            {
                if (SpellID != args[0])
                {
                    CastStartTime = DateTime.Now;
                    SpellID = args[0];
                    //SpellName = GameManager.LocalPlayer.CurrentCast.Name;
                    CurrentThresshold = 1;
                    MaxThresshold = args[1];
                }
                return true;
            }

            public static bool ThresholdUpdate(LuaEventArgument[] args)
            {
                if (SpellID == args[0])
                    CurrentThresshold = args[1];
                return true;
            }

            public static bool ThresholdClear(LuaEventArgument[] args)
            {
                if (SpellID == args[0])
                {
                    CastStartTime = DateTime.MinValue;
                    SpellID = 0;
                    SpellName = "";
                    CurrentThresshold = 0;
                    MaxThresshold = 0;
                }
                return true;
            }

            public static string UseGadget()
            {
                #region Gadgets

                #region Gadget: Absorb
                if (SpellController.CanCast("Absorb"))
                    return "Absorb";
                #endregion

                #region Gadget: Barrier
                if (SpellController.CanCast("Barrier"))
                    return "Barrier";
                #endregion

                #region Gadget: Bellow
                if (SpellController.CanCast("Bellow"))
                    return "Bellow";
                #endregion

                #region Gadget: Berserk
                if (SpellController.CanCast("Berserk"))
                    return "Berserk";
                #endregion

                #region Gadget: Bloodscent
                if (SpellController.CanCast("Bloodscent"))
                    return "Bloodscent";
                #endregion

                #region Gadget: Break
                if (SpellController.CanCast("Break"))
                    return "Break";
                #endregion

                #region Gadget: Break Free
                if (SpellController.CanCast("Break Free"))
                    return "Break Free";
                #endregion

                #region Gadget: Burly
                if (SpellController.CanCast("Burly"))
                    return "Burly";
                #endregion

                #region Gadget: Buffer
                if (SpellController.CanCast("Buffer"))
                    return "Buffer";
                #endregion

                #region Gadget: Cleanse
                if (SpellController.CanCast("Cleanse"))
                    return "Cleanse";
                #endregion

                #region Gadget: Cruelty
                if (SpellController.CanCast("Cruelty"))
                    return "Cruelty";
                #endregion

                #region Gadget: Defender
                if (SpellController.CanCast("Defender"))
                    return "Defender";
                #endregion

                #region Gadget: Destruction
                if (SpellController.CanCast("Destruction"))
                    return "Destruction";
                #endregion

                #region Gadget: Dodger
                if (SpellController.CanCast("Dodger"))
                    return "Dodger";
                #endregion

                #region Gadget: Disabler
                if (SpellController.CanCast("Disabler"))
                    return "Disabler";
                #endregion

                #region Gadget: Disintegrate
                if (SpellController.CanCast("Disintegrate"))
                    return "Disintegrate";
                #endregion

                #region Gadget: Dissolve
                if (SpellController.CanCast("Dissolve"))
                    return "Dissolve";
                #endregion

                #region Gadget: E.M.P.
                if (SpellController.CanCast("E.M.P."))
                    return "E.M.P.";
                #endregion

                #region Gadget: Encase
                if (SpellController.CanCast("Encase"))
                    return "Encase";
                #endregion

                #region Gadget: Erudite
                if (SpellController.CanCast("Erudite"))
                    return "Erudite";
                #endregion

                #region Gadget: Exodus Sands
                if (SpellController.CanCast("Exodus Sands"))
                    return "Exodus Sands";
                #endregion

                #region Gadget: Ferocity
                if (SpellController.CanCast("Ferocity"))
                    return "Ferocity";
                #endregion

                #region Gadget: Finesse
                if (SpellController.CanCast("Finesse"))
                    return "Finesse";
                #endregion

                #region Gadget: Fortitude
                if (SpellController.CanCast("Fortitude"))
                    return "Fortitude";
                #endregion

                #region Gadget: Giant's Growth
                if (SpellController.CanCast("Giant's Growth"))
                    return "Giant's Growth";
                #endregion

                #region Gadget: Guard
                if (SpellController.CanCast("Guard"))
                    return "Guard";
                #endregion

                #region Gadget: Guiding Light
                if (SpellController.CanCast("Guiding Light"))
                    return "Guiding Light";
                #endregion

                #region Gadget: Heal
                if (SpellController.CanCast("Heal"))
                    return "Heal";
                #endregion

                #region Gadget: Healer's Gift
                if (SpellController.CanCast("Healer's Gift"))
                    return "Healer's Gift";
                #endregion

                #region Gadget: Intensity
                if (SpellController.CanCast("Intensity"))
                    return "Intensity";
                #endregion

                #region Gadget: Juggernaut
                if (SpellController.CanCast("Juggernaut"))
                    return "Juggernaut";
                #endregion

                #region Gadget: Mirage Relic
                if (SpellController.CanCast("Mirage Relic"))
                    return "Mirage Relic";
                #endregion

                #region Gadget: Mist of the Oasis
                if (SpellController.CanCast("Mist of the Oasis"))
                    return "Mist of the Oasis";
                #endregion

                #region Gadget: Parry
                if (SpellController.CanCast("Parry"))
                    return "Parry";
                #endregion

                #region Gadget: Power
                if (SpellController.CanCast("Power"))
                    return "Power";
                #endregion

                #region Gadget: Power Reserves
                if (SpellController.CanCast("Power Reserves"))
                    return "Power Reserves";
                #endregion

                #region Gadget: Powerup
                if (SpellController.CanCast("Powerup"))
                    return "Powerup";
                #endregion

                #region Gadget: Punisher
                if (SpellController.CanCast("Punisher"))
                    return "Punisher";
                #endregion

                #region Gadget: Precision
                if (SpellController.CanCast("Precision"))
                    return "Precision";
                #endregion

                #region Gadget: Protection
                if (SpellController.CanCast("Protection"))
                    return "Protection";
                #endregion

                #region Gadget: Protection of the Voidstriker
                if (SpellController.CanCast("Protection of the Voidstriker"))
                    return "Protection of the Voidstriker";
                #endregion

                #region Gadget: Pursuant
                if (SpellController.CanCast("Pursuant"))
                    return "Pursuant";
                #endregion

                #region Gadget: Rally
                if (SpellController.CanCast("Rally"))
                    return "Rally";
                #endregion

                #region Gadget: Recovery
                if (SpellController.CanCast("Recovery"))
                    return "Recovery";
                #endregion

                #region Gadget: Reinforce
                if (SpellController.CanCast("Reinforce"))
                    return "Reinforce";
                #endregion

                #region Gadget: Replenish
                if (SpellController.CanCast("Replenish"))
                    return "Replenish";
                #endregion

                #region Gadget: Reserves
                if (SpellController.CanCast("Reserves"))
                    return "Reserves";
                #endregion

                #region Gadget: Salvation
                if (SpellController.CanCast("Salvation"))
                    return "Salvation";
                #endregion

                #region Gadget: Savagery
                if (SpellController.CanCast("Savagery"))
                    return "Savagery";
                #endregion

                #region Gadget: Second Wind
                if (SpellController.CanCast("Second Wind"))
                    return "Second Wind";
                #endregion

                #region Gadget: Severity
                if (SpellController.CanCast("Severity"))
                    return "Severity";
                #endregion

                #region Gadget: Shield Charge
                if (SpellController.CanCast("Shield Charge"))
                    return "Shield Charge";
                #endregion

                #region Gadget: Shredder
                if (SpellController.CanCast("Shredder"))
                    return "Shredder";
                #endregion

                #region Gadget: Stasis
                if (SpellController.CanCast("Stasis"))
                    return "Stasis";
                #endregion

                #region Gadget: Tenacity
                if (SpellController.CanCast("Tenacity"))
                    return "Tenacity";
                #endregion

                #region Gadget: True Strike
                if (SpellController.CanCast("True Strike"))
                    return "True Strike";
                #endregion

                #region Gadget: Unbreakable
                if (SpellController.CanCast("Unbreakable"))
                    return "Unbreakable";
                #endregion

                #region Gadget: Unleash
                if (SpellController.CanCast("Unleash"))
                    return "Unleash";
                #endregion

                #region Gadget: Vanish
                if (SpellController.CanCast("Vanish"))
                    return "Vanish";
                #endregion

                #region Gadget: Wisdom
                if (SpellController.CanCast("Wisdom"))
                    return "Wisdom";
                #endregion

                #endregion

                return null;
            }
        }
    }
}
