﻿using Agility.Routines;
using Buddy.Common;
using Buddy.Wildstar.Engine;
using log4net;
using MahApps.Metro.Controls;
using Newtonsoft.Json;
using System;
using System.Linq;
using System.IO;
using System.Windows;
using System.Windows.Markup;
using System.Collections.Generic;
using Buddy.Wildstar.Game;

namespace Agility.Settings
{
    class AgilitySettings : JsonSettings
    {
        private Flyout mainFlyout;
        private MetroWindow MainWindow
        {
            get
            {
                return (MetroWindow)Buddy.Wildstar.App.Current.MainWindow;
            }
        }
        private ILog Log = LogManager.GetLogger("Agility.AgilitySettings");

        // Constructor.
        public AgilitySettings()
            : base(GetSettingsFilePath(SettingsPath, "Agility", Buddy.Wildstar.Game.GameManager.LocalPlayer.Name, "Settings.json"))
        {
            using (var fs = new FileStream(@"Bots\Agility\Settings\AgilitySettings.xaml", FileMode.Open))
            {
                this.mainFlyout = (Flyout)XamlReader.Load(fs);
                (this.mainFlyout as FrameworkElement).DataContext = this;
            }
        }

        // Singleton.
        private static AgilitySettings _instance;
        public static AgilitySettings Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new AgilitySettings();
        
                return _instance;
            }
        }

        // Methods.
        public void Toggle()
        {
            if (this.mainFlyout.IsOpen)
                this.Hide();
            else
                this.Show();
        }
        private void Add()
        {
            if (!MainWindow.Flyouts.Items.Contains(this.mainFlyout))
                MainWindow.Flyouts.Items.Add(this.mainFlyout);
        }
        private void Remove()
        {
            if (MainWindow.Flyouts.Items.Contains(this.mainFlyout))
                MainWindow.Flyouts.Items.Remove(this.mainFlyout);
        }
        private void Show()
        {
            if (!MainWindow.Flyouts.Items.Contains(this.mainFlyout))
                this.Add();

            this.mainFlyout.IsOpen = true;
        }
        private void Hide()
        {
            this.mainFlyout.IsOpen = false;
        }

        // General.

        #region AutoLoot Binding

        private bool _AutoLoot = true;
        public bool AutoLoot
        {
            get { return _AutoLoot; }
            set
            {
                if (value.Equals(_AutoLoot))
                    return;

                _AutoLoot = value;
                Log.Debug("Auto Loot : " + _AutoLoot.ToString());
                NotifyPropertyChanged(() => AutoLoot);
                NotifyPropertyChanged(() => AutoLootExtend);
                base.Save();
            }
        }

        [JsonIgnore]
        public string AutoLootExtend
        {
            get
            {
                if (_AutoLoot)
                    return "Visible";
                else
                    return "Collapsed";
            }
            set
            {
            }
        }

        #endregion

        #region GroupAutoLoot Binding

        private bool _GroupAutoLoot = true;
        public bool GroupAutoLoot
        {
            get { return _GroupAutoLoot; }
            set
            {
                if (value.Equals(_GroupAutoLoot))
                    return;

                _GroupAutoLoot = value;
                Log.Debug("Group Auto Loot : " + _GroupAutoLoot.ToString());
                NotifyPropertyChanged(() => GroupAutoLoot);
                base.Save();
            }
        }

        #endregion

        #region AutoCSI Binding

        private bool _AutoCSI = true;
        public bool AutoCSI
        {
            get { return _AutoCSI; }
            set
            {
                if (value.Equals(_AutoCSI))
                    return;

                _AutoCSI = value;
                Log.Debug("Auto CSI : " + _AutoCSI.ToString());
                NotifyPropertyChanged(() => AutoCSI);
                base.Save();
            }
        }

        #endregion

        #region AutoBreak Binding

        private bool _AutoBreak = true;
        public bool AutoBreak
        {
            get { return _AutoBreak; }
            set
            {
                if (value.Equals(_AutoBreak))
                    return;

                _AutoBreak = value;
                Log.Debug("Auto Break : " + _AutoBreak.ToString());
                NotifyPropertyChanged(() => AutoBreak);
                base.Save();
            }
        }

        #endregion

        // Hotkeys.

        #region ComboBox List

        //private List<Type> _Routines;
        [JsonIgnore]
        public List<String> Routines
        {
            get 
            {
                return AppDomain.CurrentDomain.GetAssemblies()
                    .SelectMany(s => s.GetTypes())
                    .Where(p => p.IsClass && p.Namespace == "Agility.Routines")
                    .Select(q => q.FullName)
                    .ToList();
            }
        }
        

        #endregion

        #region Routine 1

        private string _Routine1 = "Agility.Routines."+GameManager.LocalPlayer.Class+"_Default";
        public string Routine1
        {
            get { return _Routine1; }
            set
            {
                if (value.Equals(_Routine1))
                    return;

                _Routine1 = value;
                Log.Debug("Routine 1 : " + _Routine1);
                NotifyPropertyChanged(() => Routine1);
                base.Save();
            }
        }

        #endregion

        #region Routine 2

        private string _Routine2 = "Agility.Routines." + GameManager.LocalPlayer.Class + "_AutoAttack";
        public string Routine2
        {
            get { return _Routine2; }
            set
            {
                if (value.Equals(_Routine2))
                    return;

                _Routine2 = value;
                Log.Debug("Routine 2 : " + _Routine2);
                NotifyPropertyChanged(() => Routine2);
                base.Save();
            }
        }

        #endregion

        #region Routine 3

        private string _Routine3 = "Agility.Routines." + GameManager.LocalPlayer.Class + "_Stun";
        public string Routine3
        {
            get { return _Routine3; }
            set
            {
                if (value.Equals(_Routine3))
                    return;

                _Routine3 = value;
                Log.Debug("Routine 3 : " + _Routine3);
                NotifyPropertyChanged(() => Routine3);
                base.Save();
            }
        }

        #endregion

        #region Routine 4

        private string _Routine4 = "Agility.Routines.DefaultRoutine";
        public string Routine4
        {
            get { return _Routine4; }
            set
            {
                if (value.Equals(_Routine4))
                    return;

                _Routine4 = value;
                Log.Debug("Routine 4 : " + _Routine4);
                NotifyPropertyChanged(() => Routine4);
                base.Save();
            }
        }

        #endregion

        #region Routine 5

        private string _Routine5 = "Agility.Routines.DefaultRoutine";
        public string Routine5
        {
            get { return _Routine5; }
            set
            {
                if (value.Equals(_Routine5))
                    return;

                _Routine5 = value;
                Log.Debug("Routine 5 : " + _Routine5);
                NotifyPropertyChanged(() => Routine5);
                base.Save();
            }
        }

        #endregion

        #region Routine 6

        private string _Routine6 = "Agility.Routines.DefaultRoutine";
        public string Routine6
        {
            get { return _Routine6; }
            set
            {
                if (value.Equals(_Routine6))
                    return;

                _Routine6 = value;
                Log.Debug("Routine 6 : " + _Routine6);
                NotifyPropertyChanged(() => Routine6);
                base.Save();
            }
        }

        #endregion

        #region Routine 7

        private string _Routine7 = "Agility.Routines.DefaultRoutine";
        public string Routine7
        {
            get { return _Routine7; }
            set
            {
                if (value.Equals(_Routine7))
                    return;

                _Routine7 = value;
                Log.Debug("Routine 7 : " + _Routine7);
                NotifyPropertyChanged(() => Routine7);
                base.Save();
            }
        }

        #endregion

        #region Routine 8

        private string _Routine8 = "Agility.Routines.DefaultRoutine";
        public string Routine8
        {
            get { return _Routine8; }
            set
            {
                if (value.Equals(_Routine8))
                    return;

                _Routine8 = value;
                Log.Debug("Routine 8 : " + _Routine8);
                NotifyPropertyChanged(() => Routine8);
                base.Save();
            }
        }

        #endregion


        // Debug.

        #region IsForegroundRequired Binding

        private bool _IsForegroundRequired = true;
        public bool IsForegroundRequired
        {
            get { return _IsForegroundRequired; }
            set
            {
                if (value.Equals(_IsForegroundRequired))
                    return;

                _IsForegroundRequired = value;
                Log.Debug("IsForegroundRequired : " + _IsForegroundRequired.ToString());
                NotifyPropertyChanged(() => IsForegroundRequired);
                base.Save();
            }
        }

        #endregion

        #region TicksPerSecond Binding

        private byte _TPS = 10;
        public byte TPS
        {
            get { return _TPS; }
            set
            {
                if (value.Equals(_TPS))
                    return;

                _TPS = value;
                GameEngine.BotPulsator.TicksPerSecond = value;
                Log.Debug("TPS : " + _TPS.ToString());
                NotifyPropertyChanged(() => TPS);
                base.Save();
            }
        }

        #endregion

    }
}